/*
 * file:   man1d.h
 * author: Hinke Osinga - University of Bristol
 * date:   October 1997
 *
 * edited: James England - University of Bristol
 * date:   November 2002
 */

#ifndef MAN1D_H
#define MAN1D_H

#include "memory.h"
#include "fixed.h"
#include "man1d_list.h"


#define    BOTH       0
#define    POSITIVE   1
#define    NEGATIVE   2


struct Man1D_DataS{
  memory             memory;
  int                v_dim;
  int                p_dim;
  int                map_per;
  int                man_type;
  double             arclength;
  int                side;
  int                method;
  int                extend;
  double             dinit, dmin;
  double             stabarc, unstabarc;
  double             amin, amax;
  double             damin, damax;
  double             fudge;
  double             converge;
  double 	     bierror;
  int                max_iter;
  double             *fixpt;
  double             **finpt;  /* points finshed at */
  double             **extpt;  /* points to extend from */
  double             *p, *x, *work;         /* some workspace; 
					       points in phase space */
  double             *evalues, **evectors;
  ONED_MAN           mfld;
  struct Prop_DataS  prop_cntl;
  int                num_extpt;
  int                explicit_switch;
  int                approx_switch;
  int                jacobian_switch;
};

/* man1d_install.c */
void man1d_install(void);
void man1d_clear(void);

/* man1d_load.c */
int  man1d_load(struct Man1D_DataS *s);
int  release_man1d(struct Man1D_DataS *s);
void iteration_set_up(struct Prop_DataS *iter_cntl);

/* man1d_linear.c */
int man1d_linear(struct Man1D_DataS *s, struct Fixpt_DataS *fp, 
		 double *params);

/* man1d_acc.c */
double Norm(int dim, double *p, double *q, double *qlift, 
	    Manifold *proj);
void   Check_Point(double *p1, double *p2, double *p3, int dim, 
		   double *angle, double *dist, double *plift, 
		   Manifold *proj);
List   Find_Point(double *pnew, double *invpnew, double *p, double *invp, 
		  int dim, List inv_item, double delta, double fudge, 
		  struct Prop_DataS *iter_cntl, int *sw, int *status);

/* man1d_build.c */
int man1d_build(struct Man1D_DataS *s, int pos, double delta_start, 
		double *params);
int man1d_iterate(struct Man1D_DataS *s, double *params);
int man1d_grow(struct Man1D_DataS *s, double *params);

/* man1d_compute.c */
void man1d_compute(int st_un);
void man1ds_compute(void);
void man1du_compute(void);

/* added by J. England */

/* man1d_noinv_acc.c    */
double Find_Angle(double *p, double *pm1, int dim, struct Prop_DataS *iter_cntl);
int Side_of_Line ( double *pi, double *pim1, double *fq, int dim, double *dotProd, struct Prop_DataS *iter_cntl);
int Near_Line ( double *pi, double *pim1, double *fq, int dim, struct Prop_DataS *iter_cntl);
int Try_Point_noinv(double delta_c, double angle, double *p, double *p_test, double *fp, struct Prop_DataS *iter_cntl, int dim, int *sw);
int Search_Circle(double *pprev, double *pleft, double *pright, double *p, double delta_circle, int dim, double *pnew, double *invpnew, struct Prop_DataS *iter_cntl, int *sw, int init, struct Man1D_DataS *s);
List Check_Right_Section( List list_left, double *inv , int *sts, int dim, int new, struct Prop_DataS *iter_cntl, struct Man1D_DataS *s);
List Find_Point_alternative(double *pnew, double *invpnew, double *p,  double *prev, int dim, int new, List inv_item, double delta_circle, struct Prop_DataS *iter_cntl, int *sw, int *status, int init, struct Man1D_DataS *s);

/* man1d_noinv_build.c */

int man1d_iterate_noinv(struct Man1D_DataS *s, int pos,  double *params);
int man1d_grow_noinv(struct Man1D_DataS *s, int pos,  double *params);
int man1d_extend(struct Man1D_DataS *s, int pos, double delta_start, double *params);

#endif
