/*
 * file:   man1d_install.c
 * author: Hinke Osinga - University of Bristol
 * date:   October 1997
 *
 * edited: James England - University of Bristol
 * date:   November 2002
 */

/*
 * man1d_install(void)
 * man1d_clean(void)
 * man1d_clear(void)
 */


#include <stdio.h>

#include "pm.h"
#include "utilities.h"
#include "math_utils.h"
#include "man1d.h"


static char *MAN1D_OBJ_NAME = "Man1D";

static char *MAN1D[] = {
  "Man1D.Side",
  "Man1D.Method",
  "Man1D.Extend",
  "Man1D.Delta_Init",
  "Man1D.Stab_ARC",
  "Man1D.Unstab_ARC",
  "Man1D.A_min",
  "Man1D.A_max",
  "Man1D.DA_min",
  "Man1D.DA_max",
  "Man1D.Abs_MIN",
  "Man1D.Fudge",
  "Man1D.Converge",
  "Man1D.Bi_error",
  "Man1D.Max_iter",
  "Man1D.clear_man1d",
  "Man1D.compute_man1ds",
  "Man1D.compute_man1du",
  "Man1D.Reset"
};

typedef enum{
  SIDE, METHOD, EXTEND, DELTA_INIT, STAB_ARC, UNSTAB_ARC, A_MIN, A_MAX, DA_MIN, DA_MAX, ABS_MIN, FUDGE, CONVERGE, BIERROR, MAXITER, CLEAR_MAN1D, COMPUTE_MAN1DS, COMPUTE_MAN1DU, RESET
} MAN1D_t;


void man1d_install()
{
  void man1d_clear(), man1d_compute(), man1d_reset();

  /* create elements for panel values in the postmaster */
  pm(CREATE_OBJ,  MAN1D_OBJ_NAME, 
     CREATE_ELEM, MAN1D[SIDE], INT,
     CREATE_ELEM, MAN1D[METHOD], INT,
     CREATE_ELEM, MAN1D[EXTEND], INT,
     CREATE_ELEM, MAN1D[DELTA_INIT], DBL,
     CREATE_ELEM, MAN1D[STAB_ARC], DBL,
     CREATE_ELEM, MAN1D[UNSTAB_ARC], DBL,
     CREATE_ELEM, MAN1D[A_MIN], DBL,
     CREATE_ELEM, MAN1D[A_MAX], DBL,
     CREATE_ELEM, MAN1D[DA_MIN], DBL,
     CREATE_ELEM, MAN1D[DA_MAX], DBL,
     CREATE_ELEM, MAN1D[ABS_MIN], DBL,
     CREATE_ELEM, MAN1D[FUDGE], DBL,
     CREATE_ELEM, MAN1D[CONVERGE], DBL,
     CREATE_ELEM, MAN1D[BIERROR], DBL,
     CREATE_ELEM, MAN1D[MAXITER], INT,
     CREATE_ELEM, MAN1D[CLEAR_MAN1D], FNCT,
     CREATE_ELEM, MAN1D[CLEAR_MAN1D], FNCT,
     CREATE_ELEM, MAN1D[COMPUTE_MAN1DS], FNCT,
     CREATE_ELEM, MAN1D[COMPUTE_MAN1DU], FNCT,
     CREATE_ELEM, MAN1D[RESET], FNCT,
     NULL);

  /* initialize the function pointers in the postmaster */
  pm(INIT, MAN1D[CLEAR_MAN1D],
     PUT, MAN1D[CLEAR_MAN1D], man1d_clear,
     INIT, MAN1D[COMPUTE_MAN1DS],
     PUT, MAN1D[COMPUTE_MAN1DS], man1ds_compute,
     INIT, MAN1D[COMPUTE_MAN1DU],
     PUT, MAN1D[COMPUTE_MAN1DU], man1du_compute,
     INIT, MAN1D[RESET],
     PUT, MAN1D[RESET], man1d_reset,
     NULL);
}

void man1d_reset()
{
  pm(PUT, MAN1D[SIDE], 0, NULL);
  pm(PUT, MAN1D[METHOD], 1, NULL);
  pm(PUT, MAN1D[DELTA_INIT], 1.0e-3, NULL);
  pm(PUT, MAN1D[STAB_ARC], 10.0, NULL);
  pm(PUT, MAN1D[UNSTAB_ARC], 10.0, NULL);
  pm(PUT, MAN1D[A_MIN], 0.2, NULL);
  pm(PUT, MAN1D[A_MAX], 0.3, NULL);
  pm(PUT, MAN1D[DA_MIN], 1.0e-6, NULL);
  pm(PUT, MAN1D[DA_MAX], 1.0e-5, NULL);
  pm(PUT, MAN1D[ABS_MIN], 1.0e-4, NULL);
  pm(PUT, MAN1D[FUDGE], 0.2, NULL);
  pm(PUT, MAN1D[CONVERGE], 1.0e-8, NULL);
  pm(PUT, MAN1D[BIERROR], 1.0e-6, NULL);
  pm(PUT, MAN1D[MAXITER], 1, NULL);
}

void man1d_clear(void)
{
  memory  m;
  int     manifold_test();

  m = (memory) pm(GET, "Memory.Fixed", NULL);
  memory_clear_selected_flows(m, manifold_test);
  pm(EXEC, "Memory.Count", NULL);
}




