/*
 * file:   man1d_linear.c
 * author: Hinke Osinga - University of Bristol
 * date:   November 1997
 */

/*
 * man1d_linear(struct Man1D_DataS *s, struct Fixpt_DataS *fp, 
 *              double *params)
 */

#include "flow.h"
#include "eigen.h"
#include "fixed.h"
#include "../fixed/fixed_local.h"      
                                /* replace $DSTOOL with full address */
#include "man1d.h"


int man1d_linear(struct Man1D_DataS *s, struct Fixpt_DataS *fp, 
		 double *params)
/* recompute Jacobian, eigenvalues and eigenvectors */
{
  int  status = 0, j, k, var_dim;

  var_dim = s->v_dim - 1;
  fp->x1[var_dim] = s->fixpt[var_dim];
  for (j = 0; j < var_dim; j++){
    fp->x1[j] = s->fixpt[j];
    fp->x2[j] = fp->fd_step;
  }
  for (j = 0; j < s->p_dim; j++)
    fp->prop_cntl.parameters[j] = params[j];

  /* For maps Jac = F(x) - Id */
  fp_jac(fp, 0);
  for (j = 0; j < var_dim; j++)
    fp->jacobian[j][j] += 1.0;

  /* use converted Fortran routines for eigensystem computation */
  for (j = 0; j < var_dim; j++)
    for (k = 0; k < var_dim; k++)
      fp->jact[j+1][k+1] = fp->jacobian[j][k];
  status = rg(var_dim, var_dim, fp->jact, fp->eval[0]-1, fp->eval[1]-1, 1,
	      fp->evectors, fp->indx, fp->x3);
  for (j = 0; j < var_dim; j++){
    s->evalues[j] = fp->eval[0][j];
    for (k = 0; k < var_dim; k++)
      s->evectors[j][k] = fp->evectors[j+1][k+1];
  }

  return status;
}

