/*
 * file:   man1d_load.c
 * author: Hinke Osinga - University of Bristol
 * date:   November 1997
 *
 * edited: James England - University of Bristol
 * date:   November 2002
 */

/*
 * man1d_load(struct Man1D_DataS *s)
 * release_man1d(struct Man1D_DataS *s)
 * iteration_set_up(struct Prop_DataS *iter_cntl)
 */


#include "pm.h"
#include "utilities.h"
#include "math_utils.h"
#include "constants.h"
#include "defaults.h"
#include "symbols.h"
#include "flow.h"
#include "man1d_list.h"
#include "man1d.h"


int man1d_load(struct Man1D_DataS *s)
{
  int  status = NO_ERROR;

  /* load info about the model */
  s->memory = (memory) pm(GET, "Memory.Fixed", NULL);
  s->v_dim = *((int *) pm(GET, "Model.Varb_Dim", NULL));
  s->p_dim = *((int *) pm(GET, "Model.Param_Dim", NULL));
  s->map_per = *((int *) pm(GET, "Fixed.Map_Period", NULL));

  /* load info from the panels */
  s->side = *((int *) pm(GET, "Man1D.Side", NULL));
  s->method = *((int *) pm(GET, "Man1D.Method", NULL));
  s->extend = *((int *) pm(GET, "Man1D.Extend", NULL));
  s->dinit = *((double *) pm(GET, "Man1D.Delta_Init", NULL));
  s->stabarc = *((double *) pm(GET, "Man1D.Stab_ARC", NULL));
  s->unstabarc = *((double *) pm(GET, "Man1D.Unstab_ARC", NULL));
  s->amin = *((double *) pm(GET, "Man1D.A_min", NULL));
  s->amax = *((double *) pm(GET, "Man1D.A_max", NULL));
  s->damin = *((double *) pm(GET, "Man1D.DA_min", NULL));
  s->damax = *((double *) pm(GET, "Man1D.DA_max", NULL));
  s->dmin = *((double *) pm(GET, "Man1D.Abs_MIN", NULL));
  s->fudge = *((double *) pm(GET, "Man1D.Fudge", NULL));
  s->converge = *((double *) pm(GET, "Man1D.Converge", NULL));
  s->bierror = *((double *) pm(GET, "Man1D.Bi_error", NULL));
  s->max_iter = *((int *) pm(GET, "Man1D.Max_iter", NULL));

  s->num_extpt=0;

  status = mfld_load(&(s->mfld), s->v_dim-1);

  /* setup propagation structure */
  s->prop_cntl.direction = FORWARD;
  s->prop_cntl.iterations = 1;
  s->prop_cntl.start_to_save = 0;
  s->prop_cntl.time_step = *((double *) pm( GET, "Flow.Stepsize", NULL));
  s->prop_cntl.prop_mode = PROP_NSTEP;

  s->prop_cntl.f_iter = s->prop_cntl.f_skip = 
    *((int *) pm(GET, "Flow.Skip_Size", NULL));
  s->prop_cntl.table_color = -1; /* system color */
  
  /* allocate memory for the pointers */
  if (allocate_integ(&(s->prop_cntl), 1)) status = -1;
  if (!(s->fixpt = dvector(0,s->v_dim-1))) status = -1;
  if (!(s->p = dvector(0,s->v_dim-1))) status = -1;
  if (!(s->x = dvector(0,s->v_dim-1))) status = -1;
  if (!(s->work = dvector(0,s->v_dim-1))) status = -1;
  if (comp_alloc(&(s->prop_cntl))) status = -1;
  else 
    comp_def_setup(&(s->prop_cntl));

  /* eigenvalues of saddles are real */
  if (!(s->evalues = dvector(0,s->v_dim-1))) status = -1;
  if (!(s->evectors = dmatrix(0,s->v_dim-1, 0, s->v_dim-1))) status = -1;

  if (status == -1){
    system_mess_proc(1, "man1d_load: Memory Allocation Failure \n");
    release_man1d(s);
  }

  return status;
}

int release_man1d(struct Man1D_DataS *s)
{
  int status = NO_ERROR;

  s->num_extpt=0;

  /* free memory allocated in man1d_load */
  comp_free(&(s->prop_cntl));
  release_integ(&(s->prop_cntl), 1);

  free_dvector(s->fixpt, 0, s->v_dim-1);
  free_dvector(s->p, 0, s->v_dim-1);
  free_dvector(s->x, 0, s->v_dim-1);
  free_dvector(s->work, 0, s->v_dim-1);
  free_dvector(s->evalues, 0, s->v_dim-1);
  free_dmatrix(s->evectors, 0, s->v_dim-1, 0, s->v_dim-1);

  mfld_release(&(s->mfld));

  return status;
}


void iteration_set_up(struct Prop_DataS *iter_cntl)
{
  if (iter_cntl->mapping_flag){
    iter_cntl->inverse_flag = 
      *((int *) pm(GET, "Model.Inverse_Flag", NULL));
    iter_cntl->explicit_inverse =
      *((int *) pm(GET, "Model.Explicit_Inverse", NULL));
    iter_cntl->jacobian_flag =
      *((int *) pm(GET, "Model.Jacobian_Flag", NULL));
    iter_cntl->inverse = (int (*)()) pm(GET, "Model.Inverse", NULL);
    iter_cntl->dxdt = (int (*)()) pm(GET, "Model.DfDt", NULL);
    iter_cntl->f_iter = *((int *) pm (GET, "Flow.Skip_Size", NULL));
  }
  else
    system_mess_proc(1, "This is not a map, stop!\n");
}
