/*
 * file:   man1d_noinv_build.c
 * author: James England - University of Bristol
 * date:   November 2002
 */

/* 
 * man1d_iterate_noinv(struct Man1D_DataS *s, int pos, double *params)
 * man1d_grow_noinv(struct Man1D_DataS *s, int pos, double *params)
 * man1d_extend(struct Man1D_DataS *s, int pos, double delta_start, *params)
 */

#include <math.h>
#include <stdlib.h>
#include "complib.h"
#include "utilities.h"
#include "math_utils.h"
#include "manifold.h"
#include "man1d_funct.h"
#include "man1d_list.h"
#include "plot.h"
#include "man1d.h"
#include "pm.h"


int man1d_iterate_noinv(struct Man1D_DataS *s, int pos, double *params)
{
  int        var_dim, i;
  int        status = 0;
  int        sw[3]; 
  double     alpha, delta, delta_circle, est_delta, start_delta;
  double     end_delta, theta;
  int        bool_item[5]; 
  /* 0- right side, 1- endTooshort 2-startTooLong, 3-found, 4-next */
  int 	     delta_error=0;
  double     *p, *fp, *pbuur, *p_test, *v, *w ;
  double     eval;
  double     dotvw;
  List       item, inv_item, new_inv_item;
  
  printf("Growing initial segments - Delta_k not known.\n");
  
  sw[0] = s->explicit_switch;
  sw[1] = s->approx_switch;
  sw[2] = s->jacobian_switch;
  var_dim = s->v_dim-1;
  
  p = dvector(0, var_dim);
  fp = dvector(0, var_dim);
  pbuur = dvector(0, var_dim);
  p_test = dvector(0, var_dim);
  v = dvector(0, var_dim);
  w = dvector(0, var_dim);
  
  for (i = 0 ;i < 5 ; i++){
    bool_item[i]=0;
  }
  
  
  eval = s->evalues[pos];
  /*  if(eval < 0){
    eval = eval * eval;
    neg_eval=1;
    }*/
  
  item = s->mfld.tail;
  inv_item = item->invleft;
  for (i = 0; i < var_dim; i++){
    pbuur[i] = item->prev->data[i];
    p[i] = item->data[i];
  }
  
  do {
    
    est_delta = s->mfld.arclength*( 1/eval  - 1); /* estimate for delta_k */ 
    end_delta = 3*est_delta/4;
    start_delta = est_delta;                      /* range for delta_k */
    
    theta = Find_Angle(p, pbuur, var_dim, &(s->prop_cntl));
    
    status = Try_Point_noinv(end_delta, theta, p, p_test, 
			     fp, &(s->prop_cntl), var_dim, sw);
    bool_item[1] = Near_Line(p, pbuur, fp, var_dim, &(s->prop_cntl));
    
    do{
      /* do while the end delta is too short */
      end_delta = end_delta*2;
      status = Try_Point_noinv(end_delta, theta, p, p_test, 
			       fp, &(s->prop_cntl), var_dim, sw);
      
      bool_item[1] = Near_Line(p, pbuur, fp, var_dim, &(s->prop_cntl));
      
    } while (bool_item[1]==1);
    
    do{
      /* do while the start delta is too long */
      start_delta = start_delta/2;
      status = Try_Point_noinv(start_delta, theta, p, p_test, 
			       fp, &(s->prop_cntl), var_dim, sw);

      bool_item[2] = (Near_Line(p, pbuur, fp, var_dim, &(s->prop_cntl)))*-1 + 1;
    } while (bool_item[2]==1);

    while(bool_item[3]!=1 && bool_item[1]!=1){ 
      /* while delta not found */
	delta_circle = (end_delta + start_delta)/2;
	status = Try_Point_noinv(delta_circle, theta, p, p_test, 
				 fp, &(s->prop_cntl), var_dim, sw);
	bool_item[0] = Near_Line(p, pbuur, fp, var_dim, &(s->prop_cntl));
	
	if( bool_item[0] == 1 ) {
	  /* Length of Delta tried too short */
	  start_delta = delta_circle;
	}
	else {
	  /* Length of Delta too long */
	  end_delta = delta_circle;
	}
	
	for (i = 0; i < var_dim; i++){
	  v[i] = fp[i] - p[i];
	  w[i] = pbuur[i] - p[i];
	}
	
	dotvw = dot(var_dim, v,w);
	
	if(  dotvw < s->bierror && dotvw > 0){
	  bool_item[3]=1;
	  //  printf("in right region dotvw=%f\n", dotvw);
	}
	if (delta_error>10 && bool_item[0]==1) {
	  /* if done 10 bisection steps and on correct side accept delta */
	  bool_item[3]=1;
	}
	delta_error++;
    }
    if(bool_item[1]==1){
      /* end delta is still too short */
      printf("ERROR - should not get here!\n");
    }
    
    bool_item[3] = 0;
    
    new_inv_item = Find_Point_alternative(p_test, fp, p, pbuur, 
					  var_dim, 1, inv_item, delta_circle, 
					  &(s->prop_cntl), sw, &status, 1, s);
   
    while(new_inv_item == NULL){
      // Find point returned NULL - therefore tried to use too large delta */
      printf("Delta chosen too large - decrease by 10 pc \n");
      delta_circle = 0.9*delta_circle;
      new_inv_item = Find_Point_alternative(p_test, fp, p, pbuur, 
					    var_dim, 0, inv_item, delta_circle, 
					    &(s->prop_cntl), sw, &status, 1, s);
    }
    
    Check_Point(pbuur, p, p_test, var_dim, &alpha, &delta, s->prop_cntl.workspace, 
		s->prop_cntl.manifold);
    
    if ((alpha < s->amax) && (alpha * delta < s->damax) &&
	(s->mfld.arclength < s->arclength) && 
	(delta > s->converge) && (status >= 0)){
      
      point_plot(p_test, params, s->prop_cntl.sys_color, s->prop_cntl.symbol);
      status = add_to_mfld(&(s->mfld), p_test, p, item);
      
      s->mfld.arclength += delta;
      item = s->mfld.tail;
      for (i = 0; i < var_dim; i++){
	pbuur[i] = p[i];
	p[i] = p_test[i];
      }

      if (status >= 0) {
	Check_Point(pbuur, p, p_test, var_dim, &alpha, &delta, 
		    s->prop_cntl.workspace, s->prop_cntl.manifold);
      }
      bool_item[4]=1;
    }
    else {
      /* failed criteria */
      bool_item[4]=0;
    }   
  } while ( bool_item[4]!=0 );
  
  printf("finished iterating initial section - arclength = %f\n", s->mfld.arclength);

  if ( status == -3) {
    /* exceeded criteria in initial section */
    status = 0;
  }

  if (delta <= s->converge) fprintf(stdout, "Convergence to fixed point.");
  
  free_dvector(p, 0, var_dim);
  free_dvector(fp, 0, var_dim);
  free_dvector(pbuur, 0, var_dim);
  free_dvector(v, 0, var_dim);
  free_dvector(w, 0, var_dim);

  return status;
}


/* start our algorithm when iterates become too far apart */
int man1d_grow_noinv(struct Man1D_DataS *s, int pos, double *params)
{
  int        var_dim, i, status = 0;
  int        sw[3], new=0; 
  double     alpha, delta, da, delta_try, half_dmin,eval;
  double     *p, *pnew, *pbuur, *invp, *invpnew;
  List       item, inv_item;
  
  sw[0] = s->explicit_switch;
  sw[1] = s->approx_switch;
  sw[2] = s->jacobian_switch;
  var_dim = s->v_dim-1;

  p = dvector(0, var_dim);
  pnew = dvector(0, var_dim);
  pbuur = dvector(0, var_dim);
  invp = dvector(0, var_dim);
  invpnew = dvector(0, var_dim);

  item = s->mfld.tail;
  for (i = 0; i <= var_dim; i++){
    pbuur[i] = item->prev->data[i];
    p[i] = item->data[i];
    invp[i] = item->inverse[i];
  }

  eval = s->evalues[pos];
  /* if(eval < 0){
    eval = eval * eval;
    neg_eval=1;
    }*/

  delta = Norm(var_dim, pbuur, p, s->prop_cntl.workspace, 
	       s->prop_cntl.manifold);
  inv_item = item->invleft;

  delta_try = delta;
  if (delta_try < s->dmin){
    delta_try = s->dmin;
  }
  half_dmin = s->dmin / 2;
  
  while ((s->mfld.arclength < s->arclength) &&
	 (delta > s->converge) && (status >= 0)){

    inv_item = Find_Point_alternative(pnew, invpnew, p, pbuur, var_dim, new, 
				      inv_item, delta_try, &(s->prop_cntl), sw, &status, 0, s);
    
    if (inv_item == NULL){
      if (status >= 0){
	inv_item = s->mfld.tail;
	delta_try /= 2;
	if (delta_try < half_dmin){
	  delta_try = half_dmin;
	}
	if (status >= 0){
	  for (i = 0; i <= var_dim; i++){
	    invpnew[i] = p[i];
          }
        }
      }
    }
    
    if (status >= 0){
      Check_Point(pbuur, p, pnew, var_dim, &alpha, &delta,
		  s->prop_cntl.workspace, s->prop_cntl.manifold);
      da = delta * alpha;
      if (((alpha < s->amax) && (da < s->damax)) ||
	  (delta < s->dmin)){
        new=0;
	if ((delta < s->dmin) &&
	    ((alpha > s->amax) || (da > s->damax))){

	  if(alpha > s->amax){
	    printf("Warning - alpha search region increased.\n");
	  }

	  fprintf(stdout, "delta = %.10f\n", delta);
	  fprintf(stdout, "alpha = %f, delta * alpha = %f; added anyway\n",
		  alpha, da);
	  new=1;
	}
	
	/* alpha is acceptable => add point to s->mfld */
	point_plot(pnew, params, s->prop_cntl.sys_color, s->prop_cntl.symbol);
	status = add_to_mfld(&(s->mfld), pnew, invpnew, inv_item);
	
	s->mfld.arclength += delta;
	
	for (i = 0; i <= var_dim; i++){
	  pbuur[i] = p[i];
	  p[i] = pnew[i];
	  invp[i] = invpnew[i];
	}
	
	if ((alpha < s->amin) && (da < s->damin)){
	  /* points are almost collinear => increase delta_try */
	  delta_try *= 2;
	}
      }
      else{
	new=1;
	/* find point closer to p => decrease delta_try */
	inv_item = s->mfld.tail->invleft;
	delta_try /= 2;
	if (delta_try < half_dmin){
	  delta_try = half_dmin;
	}
      }
    }
  }
  
  if ((status >= 0) && (delta <= s->converge)) 
    fprintf(stdout, "Convergence to fixed point: ");
  
  free_dvector(p, 0, var_dim);
  free_dvector(invp, 0, var_dim);
  free_dvector(pnew, 0, var_dim);
  free_dvector(invpnew, 0, var_dim);
  free_dvector(pbuur, 0, var_dim);

  return status;
} 

/* Extending the manifold from 'Selected points' */
int man1d_extend(struct Man1D_DataS *s, int pos, double delta_start, double *params)
{
  int var_dim = s->v_dim-1, status=0, i;
  double *p = dvector(0, var_dim);
  double *pnew = dvector(0, var_dim);
  double *pnew2 = dvector(0, var_dim);
  double *invpnew = dvector(0, var_dim);
  double *invpnew2 = dvector(0, var_dim);
  double *pprev = dvector(0, var_dim);
  int *p_color, *p_bi;
  double *p_varb, *p_param, *p_bd;
  double eval;
  int j, k, n, new=1, its=0;
  int cont=1, sw[3];
  List item, inv_item;
  memory m;
  char *name;

  name = (char *) pm(QUERY, "Memory", MEMRY, 5, NULL);

  m = (memory) pm(GET, name, NULL);
  n = memory_stored_points(m);
  printf(" Number of extension points=%d\n" ,n);

  s->num_extpt = n;

  if (m == NULL){
	return 0;
  }

  if (!(s->extpt = dmatrix(0, n, 0,s->v_dim-1))) status = -1;
  if (!(s->finpt = dmatrix(0, n, 0,s->v_dim-1))) status = -1;
  
  for (k =1; k < n+1; k++){
    
    status = memory_set_read(m, 1, 1, k,
			     NULL, NULL, NULL, NULL, NULL, NULL);
    
    if (status) return 0;
    
    /* read it and stuff into postmaster browser object */
    status = memory_read_next_point(m, &p_varb, &p_param, &p_color,
				    &p_bd, &p_bi);
    
    s->extpt[k-1][0] = p_varb[0];
    s->extpt[k-1][1] = p_varb[1];    
    
    printf("Extension point = %f,%f, \n", s->extpt[k-1][0],s->extpt[k-1][1]);
  }
  
  sw[0] = s->explicit_switch;
  sw[1] = s->approx_switch;
  sw[2] = s->jacobian_switch;
  
  eval = s->evalues[pos];

  /* if(eval < 0){
    eval = eval * eval;
    neg_eval=1;
    }*/
  
  item = s->mfld.tail;
  
  for (j = 0; j< n ; j++){
    for (i = 0; i < var_dim; i++){
      pnew[i] = s->extpt[j][i];
      s->finpt[j][i]=s->mfld.tail->data[i];
      pprev[i] = pnew[i];
    }
    
    printf("Endpoint of branch: %f,%f\n",s->finpt[j][0],s->finpt[j][1]);
    printf("\ndelta_start=%f \n\n",delta_start); 
    pprev[1] = pprev[1] - 2*sqrt(delta_start*delta_start); // should use this val?
    
    point_plot(pprev, params, s->prop_cntl.sys_color, s->prop_cntl.symbol);
    status = FInv(invpnew, pnew, &(s->prop_cntl), sw);
    
    printf("Extension point: %f,%f\n",s->extpt[j][0], s->extpt[j][1]);
    
    status = add_to_mfld(&(s->mfld), pnew, invpnew, s->mfld.tail);
    point_plot(pnew, params, s->prop_cntl.sys_color, s->prop_cntl.symbol);
    
    while(item->prev->prev!=NULL){
      item = item->prev;
    }
    
    printf("- start searching at %f, %f for mappings back onto manifold, arclength %f\n ", 
	   item->prev->data[0], item->prev->data[1], s->mfld.arclength);
   
    inv_item = item;
    status=-2;
    
    while ( cont==1 && status == -2){
      inv_item = inv_item->prev;
      inv_item = Find_Point_alternative(pnew2, invpnew2,pnew,pprev, 
					var_dim, new, inv_item, delta_start,
					&(s->prop_cntl), sw, &status, 0, s);
      if(inv_item!=NULL) {
	
	cont=1;
	if(inv_item->prev==NULL) { 
	  inv_item = inv_item->next;
	  s->prop_cntl.iter_request++;

	  if(s->prop_cntl.iter_request == s->max_iter+1) s->prop_cntl.iter_request=1;

	  printf("Swapping to iterate %d.(in initial section)\n", s->prop_cntl.iter_request);
	  // neg_eval = -1*neg_eval +1;
          its++;
          if(its==s->max_iter+1) status=-1;	  
	}
      }
      else printf(" extend:  inv_item NULL returned.\n");
    }
    printf(" status=%d\n", status);
    if(inv_item!=NULL && status>=0){
      if(inv_item->prev!=NULL && status >= 0){
	printf("adding to manifold - %f, %f \n", pnew2[0], pnew2[1]);
	status = add_to_mfld(&(s->mfld), pnew2, invpnew2, inv_item);
	point_plot(pnew2, params, s->prop_cntl.sys_color, s->prop_cntl.symbol);
	printf(" Point found mapped onto manifold - %f,%f \n", pnew2[0], pnew2[1]);
	status = man1d_grow_noinv(s, pos, params);
      }
    }
    else printf(" Point not found to be mapped back onto manifold.\n");
  }
  
  free_dvector(p, 0 , var_dim);
  free_dvector(pnew, 0 , var_dim);
  free_dvector(invpnew, 0 , var_dim);
  free_dvector(pnew2, 0 , var_dim);
  free_dvector(invpnew2, 0 , var_dim);

  return status;
}
